#!/bin/sh

VERSION=1.0.0

BOARD=$1
UBOOT=$2
KERNEL=$3
DTB=$4
USERLAND=$5

BOARD_LIST="a640"

check_params ()
{
	IMAGES="$UBOOT $KERNEL $DTB $USERLAND"
	for FILE in $IMAGES; do
		if [ ! -r $FILE ]; then
			echo "ERROR: $FILE: No such file or directory"
			exit 1
		fi
	done
}

get_md5 ()
{
	md5sum $1 | awk '{ print $1 }'
}

create_install_conf ()
{
	echo "#!/bin/sh
#
# install-image configuration
#
LOGFILE=/root/install.log
SEND_LOGFILE=/root/send_log.log

#
# Files
#
BOOTLOADER_IMAGE=/images/u-boot.imx
BOOTLOADER_IMAGE_MD5=$(get_md5 $UBOOT)

KERNEL_IMAGE=/images/uImage
KERNEL_IMAGE_MD5=$(get_md5 $KERNEL)

DEVICETREE_IMAGE=/images/a640.dtb
DEVICETREE_IMAGE_MD5=$(get_md5 $DTB)

ROOTFS_ARCHIVE=/images/$(basename $USERLAND)
ROOTFS_ARCHIVE_MD5=$(get_md5 $USERLAND)"
}

clean ()
{
	if mountpoint -q $MNTDIR ; then
		umount $MNTDIR
	fi
	rm -rf $MNTDIR
}

init_images ()
{
	TEMP=$(mktemp -d)
	mkdir $TEMP/p1 $TEMP/p2

	# Init p1
	# nothing to do

	# Init p2
	tar xf core/dist.tar.gz -C $TEMP/p2
	mkdir -p $TEMP/p2/boot
	cp "$KERNEL" $TEMP/p2/boot/uImage
	cp "$DTB" $TEMP/p2/boot/a640.dtb
	cp core/images/* $TEMP/p2/images/

	create_install_conf > $TEMP/p2/images/install.conf

	cp $UBOOT $TEMP/p2/images/u-boot.imx
	cp $USERLAND $TEMP/p2/images/
	cp $KERNEL $TEMP/p2/images/uImage
	cp $DTB $TEMP/p2/images/a640.dtb
}

calc_disk_size_mb ()
{
	# Partition 1: total size of files + 1MB margin
	SIZE_DISK_P1=$(du -sm $TEMP/p1 | awk '{print $1}')
	SIZE_DISK_P1=$(( SIZE_DISK_P1 + 1 ))

	# Partition 2: total size of files + 50MB margin
	SIZE_DISK_P2=$(du -sm $TEMP/p2 | awk '{print $1}')
	SIZE_DISK_P2=$(( SIZE_DISK_P2 + 50 ))

	SIZE_DISK=$(( SIZE_DISK_P1 + SIZE_DISK_P2 ))
}

create_disk ()
{
	INSTALL_IMAGE="install-disk-sd-${BOARD}-$(date +%Y%m%d).img"
	rm -f ${INSTALL_IMAGE}

	# Make empty install image
	dd of=${INSTALL_IMAGE} count=0 seek=1 bs=${SIZE_DISK}M

	# Attach install image to loop device
	DISK_DEV_TMP=$(losetup -P -f --show ${INSTALL_IMAGE})

	# Edit partitions
	sfdisk ${DISK_DEV_TMP} <<EOF
,${SIZE_DISK_P1}M,b
,${SIZE_DISK_P2}M,83
EOF

	# Re-attach install image to loop device for partitions
	losetup -d ${DISK_DEV_TMP}
	DISK_DEV_TMP=$(losetup -P -f --show ${INSTALL_IMAGE})

	# Format partitions
	mkfs.vfat ${DISK_DEV_TMP}p1
	mkfs.ext4 -F ${DISK_DEV_TMP}p2

	MNTDIR=$(mktemp -d)

	# Copy files to partition 1
	# nothing to do

	# Copy files to partition 2
	mount ${DISK_DEV_TMP}p2 $MNTDIR
	cp -r $TEMP/p2/* $MNTDIR
	sync
	umount $MNTDIR

	rmdir $MNTDIR

	# Write u-boot image
	dd if=core/u-boot.imx of=${DISK_DEV_TMP} bs=1k seek=1 conv=fsync
	sync

	# Detach loop device
	losetup -d ${DISK_DEV_TMP}
}

usage ()
{
	echo "Install Disk Image Build Script v$VERSION

Usage:
  sudo ./build.sh BOARD UBOOT KERNEL DTB USERLAND

  BOARD: $(echo $BOARD_LIST | sed 's/ /\//g')
  UBOOT: u-boot image
  KERNEL: uImage
  DTB: Device Tree Blob image
  USERLAND: Debian userland archive

Example:
  sudo ./build.sh a640 u-boot-a600-v2018.03-at4.imx uImage-a600-v4.14-at11 armadillo-640-v4.14-at11.dtb debian-stretch-armhf-a600-20190326.tar.gz"
}

if [ $(whoami) != "root" ] ;then
	echo "ERROR: Please run script with sudo"
	exit 1
fi

if [ $# -lt 5 ]; then
	usage
	exit 1
fi

trap 'clean ; exit 1' 1 2 3 15

check_params
init_images
calc_disk_size_mb
create_disk
clean
