#!/bin/sh

. /images/install.conf

PATH=/bin:/sbin:/usr/bin:/usr/sbin

#
# internal parameters
#
DEV_EMMC=/dev/mmcblk0
BOOTLOADER_EMMC_PATH=${DEV_EMMC}
BOOTLOADER_EMMC_FORCERO_PATH=/sys/block/mmcblk0boot0/force_ro
RESERVED1_EMMC_PATH=${DEV_EMMC}p1
IMAGES_EMMC_PATH=${DEV_EMMC}p2
RESERVED2_EMMC_PATH=${DEV_EMMC}p3
BOOTLOADER_PARM_SIZE_K=512
BOOTLOADER_PARM_OFFSET_K=512
STEP=0
LED_CLASS_PATH=/sys/class/leds
NVMEM_PATH=/sys/bus/nvmem/devices/imx-ocotp0/nvmem

led_control() {
	local LED=$1
	local MODE=$2
	local INTERVAL=250
	if [ ${MODE} == "off" ]; then
		echo none > ${LED_CLASS_PATH}/${LED}/trigger
		echo 0 > ${LED_CLASS_PATH}/${LED}/brightness
	elif [ ${MODE} == "on" ]; then
		echo none > ${LED_CLASS_PATH}/${LED}/trigger
		echo 1 > ${LED_CLASS_PATH}/${LED}/brightness
	elif [ ${MODE} == "blink" ]; then
		echo timer > ${LED_CLASS_PATH}/${LED}/trigger
		echo ${INTERVAL} > ${LED_CLASS_PATH}/${LED}/delay_off
		echo ${INTERVAL} > ${LED_CLASS_PATH}/${LED}/delay_on
	fi
}

progress_running() {
	led_control "green"  "blink"
	led_control "red"    "off"
	led_control "yellow" "off"
}

progress_ok() {
	led_control "green"  "on"
	led_control "red"    "on"
	led_control "yellow" "on"
}

progress_ng() {
	led_control "green"  "off"
	led_control "red"    "blink"
	led_control "yellow" "off"
}

print_serial() {
	echo $*
}

print_step() {
	STEP=$(($STEP+1))
	print_serial ""
	print_serial ""
	print_serial "=== Step.${STEP} $* ==="
	print_serial ""
}

error_exit() {
	print_serial "##### $*"
	progress_ng
	exit 2
}

emmc_format() {
# fdisk -l -u /dev/mmcblk0
#
# Disk /dev/mmcblk0: 3791 MB, 3791650816 bytes
# 4 heads, 16 sectors/track, 115712 cylinders, total 7405568 sectors
# Units = sectors of 1 * 512 = 512 bytes
#
#         Device Boot      Start         End      Blocks  Id System
# /dev/mmcblk0p1            2048       64639       31296   b Win95 FAT32
# /dev/mmcblk0p2          314688     7405567     3545440  83 Linux
# /dev/mmcblk0p3           64640      314687      125024   b Win95 FAT32
#
# Partition table entries are not in disk order
	fdisk ${DEV_EMMC} <<EOF
o
n
p
4

+1M
n
p
1

+32M
n
p
3

+128M
n
p


t
1
b
t
2
83
t
3
b
d
4
p
w
EOF

	mkfs.vfat ${RESERVED1_EMMC_PATH}
	mke2fs -t ext4 ${IMAGES_EMMC_PATH}
	mkfs.vfat ${RESERVED2_EMMC_PATH}

}

setup_bootloader() {
	if [ ! -z "${BOOTLOADER_IMAGE_MD5}" ]; then
		echo "${BOOTLOADER_IMAGE_MD5}  ${BOOTLOADER_IMAGE}" | md5sum -c
		[ $? -eq 0 ] || return 1
	fi

	if [ -f "${BOOTLOADER_IMAGE}" ]; then
#		echo 0 > ${BOOTLOADER_EMMC_FORCERO_PATH}
		dd if=${BOOTLOADER_IMAGE} of=${DEV_EMMC} bs=1k seek=1
		sync
#		echo 1 > ${BOOTLOADER_EMMC_FORCERO_PATH}
	else
		echo "file not found ${BOOTLOADER_IMAGE}"
		return 1
	fi

	dd if=/dev/zero bs=1k count=${BOOTLOADER_PARM_SIZE_K} | \
	tr "\000" "\377" | dd of=${BOOTLOADER_EMMC_PATH} bs=1k seek=${BOOTLOADER_PARM_OFFSET_K}
	sync
}

extract_file() {
	local DEVICE=$1
	local FS=$2
	local FILE=$3
	local MD5=$4

	if [ ! -z "${FILE}" ]; then
		if [ ! -z "${MD5}" ]; then
			echo "${MD5}  ${FILE}" | md5sum -c
			[ $? -eq 0 ] || return 1
		fi

		mkdir -p /tmp/${DEVICE}
		mount -t ${FS} ${DEVICE} /tmp/${DEVICE}
		tar xzf ${FILE} -C /tmp/${DEVICE}
		sync
		umount /tmp/${DEVICE}
	else
		echo "file not found ${FILE}"
		return 1
	fi
}

copy_file() {
	local DEVICE=$1
	local FS=$2
	local FILE=$3
	local MD5=$4
	local TO=$5

	if [ ! -z "${FILE}" ]; then
		if [ ! -z "${MD5}" ]; then
			echo "${MD5}  ${FILE}" | md5sum -c
			[ $? -eq 0 ] || return 1
		fi

		mkdir -p /tmp/${DEVICE}
		mount -t ${FS} ${DEVICE} /tmp/${DEVICE}
		mkdir -p /tmp/${DEVICE}/${TO%/*}
		cp ${FILE} /tmp/${DEVICE}/${TO}
		ls /tmp/${DEVICE}/${TO}
		sync
		umount /tmp/${DEVICE}
	else
		echo "file not found ${FILE}"
		return 1
	fi
}

protect_mmc() {
	local DEVICE=$1

	local BLOCKS=$(mmc writeprotect user get ${DEVICE} | egrep -o "[0-9]+/[0-9]+" | cut -d '/' -f 1)
	mmc writeprotect user set temp 0 ${BLOCKS} ${DEVICE}
	mmc writeprotect user get ${DEVICE} | grep "Temporary Write Protection"
	[ $? -ne 0 ] && error_exit "eMMC protect error: ${DEVICE}"
}

#
# main
#
run () {
	print_serial "*+*+* Install Start!! *+*+*"
	progress_running

	print_step "Formatting eMMC"
		emmc_format
		fdisk -l /dev/mmcblk0

	print_step "Setup Partition"
		extract_file ${IMAGES_EMMC_PATH} ext4 \
			"${ROOTFS_ARCHIVE}" "${ROOTFS_ARCHIVE_MD5}"
		[ $? -ne 0 ] && error_exit "MD5 error: ${ROOTFS_ARCHIVE}"
		copy_file ${IMAGES_EMMC_PATH} ext4 \
			"${KERNEL_IMAGE}" "${KERNEL_IMAGE_MD5}"  "/boot"
		[ $? -ne 0 ] && error_exit "MD5 error: ${KERNEL_IMAGE}"
		copy_file ${IMAGES_EMMC_PATH} ext4 \
			"${DEVICETREE_IMAGE}" "${DEVICETREE_IMAGE_MD5}" "/boot"
		[ $? -ne 0 ] && error_exit "MD5 error: ${DEVICETREE_IMAGE}"

	print_step "Setup Bootloader"
		setup_bootloader

	print_step "sync"
		sync
	protect_mmc /dev/mmcblk0gp0
	print_serial "*+*+* Install Completed!! *+*+*"

	progress_ok
}

send_log() {
# Place holder to send LOGFILE from armadillo to any server
	rm ${LOGFILE}
}

# there is no way to check that run() failed or not.  we don't have
# pipestatus nor the scope for the global variables.
run 2>&1 | tee ${LOGFILE}
send_log 2>&1 | tee ${SEND_LOGFILE}
if [ -e ${LOGFILE} ]; then
	progress_ng
fi
while true; do :; done
